using System;
using System.Data.Entity;
using System.Net.Http;
using System.Threading.Tasks;
using Ninject;
using Ninject.Modules;
using TourDataManager.Entities;

namespace TourDataManager {

    
    /// <summary>
    /// Фасад для всего плагина
    /// </summary>
    public class TourDataManager {


        public FetchEstatesUseCase fetch{ get; set; }
        public Db database{ get; set; }

        private string persistentPath;

        private IKernel Container;
        
        public TourDataManager(string persistentPath){
            this.persistentPath = persistentPath;
            Container = new StandardKernel(new MyModule(persistentPath));
            
            
            fetch = Container.Get<FetchEstatesUseCase>();
            database = Container.Get<Db>();
        }

        public void Login(string email, string password){
            Container.Get<IAuthenticator>().Login(email,password, (b, s) => {
                var cookiestor = Container.Get<CookieStorage>();
                Debug.Log($"Authorization : {s}");
                Debug.Log($"Cookie count in storage : {cookiestor.Get().Count}");
                cookiestor.Save();
            });
        }

        public async Task<Estate[]> DownloadEstates(){
            try{
                var estates = await fetch.FetchEstatesFromServer();
                database.InsertEstates(estates);
                return estates;
            } catch (Exception){
                Debug.Log("Не удалось загрузить эстейты");
                return null;
            }
        }
    }

    public class MyModule : NinjectModule {

        private readonly string persistentPath;
        
        public MyModule(string persistentPath){
            this.persistentPath = persistentPath;
        }

        public override void Load(){
            var cookieStorage = new CookieStorage(persistentPath + "\\cookie.storage");
            var httpClientHandler = new HttpClientHandler();
            //CookieContainer сам запоминает полученные из ответа куки
            httpClientHandler.CookieContainer = cookieStorage.Get();
            var httpClient = new HttpClient(httpClientHandler);
            
            
            Bind<CookieStorage>().ToConstant(cookieStorage).InSingletonScope();
            Bind<HttpClientHandler>().ToConstant(httpClientHandler).InSingletonScope();
            Bind<HttpClient>().ToConstant(httpClient).InSingletonScope();
            Bind<IAuthenticator>().To<Authenticator>().InSingletonScope();
            Bind<FetchEstatesUseCase>().ToSelf().InSingletonScope();
            Bind<Db>().ToSelf().InSingletonScope();

        }
    }
}